import logging
import time
from langchain_core.prompts import ChatPromptTemplate
from langchain.agents import AgentExecutor, create_openai_functions_agent
from langchain_core.output_parsers import JsonOutputParser

from .llm_setup import initialize_llm, TokenUsageCallbackHandler
from .db_stats import log_api_call, log_agent_run
from .db_core import get_system_prompt, get_app_settings
from .agent_helpers import parse_json_output

logger = logging.getLogger(__name__)

def run_editor_agent(articles_to_merge: list[dict]):
    """
    Runs the Editor agent to merge and summarize overlapping articles.
    Args:
        articles_to_merge: A list of article dictionaries (containing 'url', 'title', 'content').
    Returns:
        A dictionary containing the 'merged_title' and 'merged_summary', or None if error.
    """
    agent_name = "Editor"
    logger.info(f"{agent_name} Agent: Starting run to merge {len(articles_to_merge)} articles.")
    if not articles_to_merge:
        logger.warning(f"{agent_name} Agent: No articles provided for merging.")
        return None

    start_time = time.time()
    run_success = False # Track success for logging
    try:
        # Fetch settings to determine which LLM provider to use
        app_settings = get_app_settings()
        editor_provider = app_settings.get('editor_llm_provider', 'openai') # Default to openai
        llm, token_callback = initialize_llm(provider=editor_provider) # Initialize LLM based on setting
        prompt_str = get_system_prompt(agent_name)
        if not prompt_str:
            logger.error(f"{agent_name} Agent: Failed to load system prompt.")
            return None

        # Prepare input string from articles
        input_text = "Please merge and summarize the following articles into a single concise news item:\n\n"
        for i, article in enumerate(articles_to_merge):
            input_text += f"Article {i+1}:\n"
            input_text += f"  Title: {article.get('title', 'N/A')}\n"
            input_text += f"  URL: {article.get('url', 'N/A')}\n"
            # Include a significant portion of content, but maybe capped
            content = article.get('content', 'N/A')
            content_preview = (content[:2000] + '...') if len(content) > 2000 else content # Limit content length
            input_text += f"  Content Preview: {content_preview}\n\n"

        # Use a chain with JSON output parser, as instructed in the default prompt
        prompt = ChatPromptTemplate.from_messages([
            ("system", prompt_str + "\n\nIMPORTANT: Your final output MUST be a single JSON object with exactly two keys: 'merged_title' and 'merged_summary'."),
            ("human", f"{input_text}\n\nGenerate the merged title and summary in the specified JSON format.")
        ])
        chain = prompt | llm | JsonOutputParser() # Langchain's parser handles JSON extraction

        # Invoke the chain
        # The input variables for the prompt are already embedded in the 'human' message string.
        # Passing an empty dict or the variables used if the prompt template expected them.
        result = chain.invoke({}, config={"callbacks": [token_callback]}) # Pass callback handler to invoke
        success = True # Assume success unless parsing fails below

        # Log the API call after invocation
        log_api_call(
            api_provider=editor_provider.capitalize(),
            service_model=llm.model_name,
            success=success, # Will be updated if parsing fails below
            caller_component=agent_name,
            input_tokens=token_callback.total_input_tokens, # Get cumulative tokens from callback
            output_tokens=token_callback.total_output_tokens, # Get cumulative tokens from callback
        )

        # --- Output Validation ---
        # JsonOutputParser should return a dict if successful
        if isinstance(result, dict) and "merged_title" in result and "merged_summary" in result:
            # Basic check for non-empty results
            if result.get("merged_title") and result.get("merged_summary"):
                 logger.info(f"{agent_name} Agent: Finished merging. Result title: {result.get('merged_title')}")
                 run_success = True # Mark as successful
                 return {"title": result["merged_title"], "summary": result["merged_summary"]}
            else:
                 logger.warning(f"{agent_name} Agent: Parsed JSON has empty title or summary. Result: {result}")
                 success = False # Mark as failed due to empty content
                 return None # Treat empty results as failure
        else:
            logger.error(f"{agent_name} Agent: Failed to parse expected JSON output or keys missing. Raw result: {result}")
            # Fallback attempt using our custom parser on the raw string output (if available)
            # This requires getting the raw string output before JsonOutputParser
            # For simplicity, we rely on JsonOutputParser for now.
            return None
        # Update log if parsing failed (though it was already logged as success)
        # This is slightly imperfect; ideally, log AFTER validation.
        # For now, we accept the initial success log might be premature if validation fails.

    except ValueError as ve: # Catch specific error for missing API key
        logger.error(f"{agent_name} Agent: Configuration error - {ve}")
        log_api_call(
            api_provider=app_settings.get('editor_llm_provider', 'N/A').capitalize(),
            service_model='N/A', success=False, caller_component=agent_name, metadata={'error': str(ve)}
        )
        return None
    except Exception as e:
        # Log the input text as well for debugging complex failures
        logger.error(f"{agent_name} Agent: Error during execution: {e}\nInput Text:\n{input_text[:500]}...", exc_info=True)
        log_api_call(
            api_provider=app_settings.get('editor_llm_provider', 'N/A').capitalize(),
            service_model=getattr(llm, 'model_name', 'N/A'), # Get model if LLM was initialized
            success=False, caller_component=agent_name,
            input_tokens=getattr(token_callback, 'total_input_tokens', 0), # Default 0 if callback unavailable
            output_tokens=getattr(token_callback, 'total_output_tokens', 0),
            metadata={'error': str(e)}
        )
        return None
    finally:
        end_time = time.time()
        log_agent_run(agent_name, start_time, end_time, run_success) # Log the run
